

local major, minor, libTable = LibStub("LibBuilder-1.0"):GetLib()
if not libTable.canLoad then return end


local interface = libTable.interface


interface.messageCache = {}
interface.tabsAvailable = false


---	Creates and returns a new Stream
--	@param	destinations	Optional; list of destinations to add to the Stream.
function interface:NewStream(...)
	local argCount = select("#", ...)
	local args = {...}
	
	local stream = libTable.classes.Stream:New()
	
	for i=1, argCount do
		local destination = self:NewDestination(args[i])
		stream:AddDestination(destination)
	end
	
	return stream
end


---	Creates and returns a new Destination.
--	@param	value	Initial value for the Destination.
function interface:NewDestination(value)
	return libTable.classes.Destination:New(value)
end


---	Internal; Checks if the client has assigned names to the tabs yet.
--	@return tabsAvailable	True if tab names are available, false otherwise.
function interface:AreTabsAvailable()
	return GetChatWindowInfo(1) ~= ""
end


--	Internal; used to dump the cache if tab names weren't available.
function interface:UPDATE_CHAT_WINDOWS()
	if #interface.messageCache > 0 then
		for i, data in ipairs(interface.messageCache) do
			data.stream:Print(data.message)
		end
	end
end


function interface:Hook()
	local frame = CreateFrame("Frame")
	frame:SetScript("OnEvent", function(...) self:UPDATE_CHAT_WINDOWS(...) end)
	frame:RegisterEvent("UPDATE_CHAT_WINDOWS")
end


function interface:QueueMessage(message, stream)
	if table.maxn(self.messageCache) == 0 then
		print(string.format("%s: caching messages", major))
	end
	table.insert(self.messageCache, { message = message, stream = stream })
end
	


interface:NewStream("LibPrint", "mcm", "debug", print):Print("loaded", major, "Revision", minor)


--	EXPERIMENTAL! SUBJECT TO CHANGE
function interface:PrintPath()
	local stack = debugstack()
	local line = string.match(stack, "\n(.-)\n")
	--	FIXME: if path gets truncated...
	local shortPath = string.match(line, "Interface\\AddOns\\(.-):")	--	FIXME: other locales?
	--print("interface:", "running", shortPath or line)
end

--interface:PrintPath()


